@@ -138,11 +138,18 @@ def muIntroduction(stationNo, typeInfoById):
     
     # 'fails' gracefully on meta and perhaps run BEFORE any but schema exist
     meta = metaOfVistA(stationNo)
-    name = "VistA" if "name" not in meta else meta["name"]
-    mu = "__{} [{}]__ ".format(name, stationNo)
-    if "cutDate" in meta:
-        mu += "was cut on _{}_ and ".format(meta["cutDate"])
-    mu += "reports <span class='yellowIt'>{:,}</span> types of data (\"File Types\"), <span class='yellowIt'>{}</span> of which are singletons (have only one record), <span class='yellowIt'>{}</span> are empty and <span class='yellowIt'>{:,}</span> are untracked. The system has a total of <span class='yellowIt'>{:,}</span> records. While the biggest type has <span class='yellowIt'>{:,}</span> records, the median number of records for types with more than one record is a lowly <span class='yellowIt'>{:,}</span>.\n\n".format(len(typeInfoById), reportAbsAndPercent(singletonCnts, len(typeInfoById)), reportAbsAndPercent(noEmptyTypes, len(typeInfoById)), noUntrackedTypes, noRecords, recordCntBiggest, median)
+    mu += "__{} [{}]__ was cut on _{}_ and reports <span class='yellowIt'>{:,}</span> types of data (\"File Types\"), <span class='yellowIt'>{}</span> of which are singletons (have only one record), <span class='yellowIt'>{}</span> are empty and <span class='yellowIt'>{:,}</span> are untracked. The system has a total of <span class='yellowIt'>{:,}</span> records. While the biggest type has <span class='yellowIt'>{:,}</span> records, the median number of records for types with more than one record is a lowly <span class='yellowIt'>{:,}</span>.\n\n".format(
+        meta["name"], 
+        stationNo, 
+        meta["cutDate"],
+        len(typeInfoById), 
+        reportAbsAndPercent(singletonCnts, len(typeInfoById)), 
+        reportAbsAndPercent(noEmptyTypes, len(typeInfoById)), 
+        noUntrackedTypes, 
+        noRecords, 
+        recordCntBiggest, 
+        median
+    )
     
     return mu
     
@@ -195,17 +202,16 @@ def expectedTypeCounts(stationNo):
                
 def main():
 
-    assert(sys.version_info >= (2,7))
-
-    if len(sys.argv) < 2:
-        print "need to specify station # ex/ 442 - exiting"
-        return
-        
-    stationNo = sys.argv[1]
+    assert sys.version_info >= (3, 4)
     
+    try:
+        stationNo = sys.argv[1]
+    except IndexError:
+        raise SystemExit("Usage _EXE_ STATIONNO")
+
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     if not os.path.isdir(userSiteDir):
-        raise Exception("Expect User Site to already exist with its basic contents")
+        raise SystemExit("Expect User Site to already exist with its basic contents")
 
     webReportDataTypesSummary(stationNo)
                  
