@@ -12,9 +12,8 @@ import numpy
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
 
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVC, muBVCOfSTProp
-from fmqlutils.cacher.cacherUtils import metaOfVistA
-from ..webReportUtils import ensureWebReportLocations, TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
-from userClassifier import UserClassifier
+from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
+from moreusrc import UserClassifier
 
 """
 Basic User Type Summary/Overview
@@ -23,39 +22,51 @@ Unlike the stopcode centric reports (off locations), these are used and institut
 etc.
 
 Requires:
-- 3.081 (Type Reduction by User: YR1 unless 'smallSOSet')
+- 3.081 (Type Reduction by User: YR1 
 - 200 (All/Indexed-reduce200)
 - ... Will move to require image logs too to round out
 
 TODO: classification is work in progress - remotes from non key institution, locals
 from non subordinate divisions etc.
-
-Note: smallSOSet setup for case where only have small sample of SOs (ie/ take all)  
 """
 def webReportUser(stationNo, deidentify=False):
 
     print("Preparing to make user report for {}{} - loading data ...".format(stationNo, " [DEIDENTIFY]" if deidentify else ""))
 
     allThere, details = checkDataPresent(stationNo, [
-        # Additional settings: force count on 
-        #     ipv4_address, remote_station_id, remote_user_ien
-        {"fileType": "3_081", "check": "YR1" if not smallSOSet else "TYPE"},
+        # Additional settings/properties:
+        # - duration (in seconds) 
+        # - force count on 
+        #     ipv4_address, remote_station_id, remote_user_ien, duration
+        {"fileType": "3_081", "check": "YR1E"},
         {"fileType": "200", "check": "ALL"} # Index
     ])
     if not allThere:
         raise Exception("Some required data is missing - {}".format(details))
-        
-    ensureWebReportLocations(stationNo)
-        
-    meta = metaOfVistA(stationNo)
-    vistaName = "VistA" if "name" not in meta else meta["name"]
 
     mu = TOP_MD_TEMPL.format("{} Users".format(stationNo), "Users")
 
-    userInfoByUserRef = reduce200(stationNo)    
-    type3_081, st3_081s = splitTypeDatas(stationNo, "3_081", reductionLabel="" if smallSOSet else "YR1", expectSubTypeProperty="user")
-    st3_081ByUserRef = dict((singleValue(st, "user"), st) for st in st3_081s if "user" in st)
+    userInfoByUserRef = reduce200(stationNo) 
+ 
+    """
+    Consider: if need to break by device (ie/ segment off SSH etc? See sniff) and
+    how to round out DOD USER breakdown ... are there any other 'CAPRI anyone' users
+    that must be broken too ie/ not just one remote user => must break out for remote
+    analysis
+    """
+    print("Loading subtypes ...")
+    type3_081, st3_081URLDs = splitTypeDatas(stationNo, "3_081", reductionLabel="YR1E", expectSubTypeProperties=["user", "level_of_assurance", "remote_app", "is_remote_200_1"])
+    sts3_081ByUserRef = defaultdict(list)
+    for st in st3_081URLDs:
+        userRef = singleValue(st, "user")
+        sts3_081ByUserRef[userRef].append(st)
     
+    # BACKWARD COMPATIBLE ... TODO: make work with ULRD (and only recombine down below 
+    # where needed)
+    print("Recombining subtypes by user (backward compatible) ...")
+    sts3_081Us = combineSubTypes(st3_081URLDs, ["user"], forceCountProps=["ipv4_address", "remote_station_id", "remote_user_ien", "duration"])
+    print("... done")
+    st3_081ByUserRef = dict((singleValue(st, "user"), st) for st in sts3_081Us if "user" in st)
     userClassifier = UserClassifier(stationNo, userInfoByUserRef, type3_081, st3_081ByUserRef)
     print("Classifying Users ...")
     classification = userClassifier.classify()
@@ -67,7 +78,7 @@ def webReportUser(stationNo, deidentify=False):
     overallFirstSODate = createDatePropInfo["firstCreateDate"].split("T")[0]
     overallLastSODate = createDatePropInfo["lastCreateDate"].split("T")[0]
         
-    BLURB_TEMPL = """Of the <span class='yellowIt'>{:,}</span> users known to {}, a minority <span class='yellowIt'>{}</span> ("__Active Users__") signed in between {} and {}, the most recent year for which data is available.
+    BLURB_TEMPL = """Of the <span class='yellowIt'>{:,}</span> users known to the system, a minority <span class='yellowIt'>{}</span> ("__Active Users__") signed in between {} and {}, {}.
 
 Most Active Users are real people but there are "__Machine Users__" for specialized VA batch applications or applications which manage end users themselves.
 
@@ -75,16 +86,16 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
 
     mu += BLURB_TEMPL.format(
         len(userInfoByUserRef),
-        "__{} [{}]__".format(vistaName, stationNo),
         reportAbsAndPercent(len(classification["activeUserRefs"]), len(userInfoByUserRef)),
         overallFirstSODate, 
-        overallLastSODate
+        overallLastSODate,
+        "the most recent year for which data is available"
     )  
     
     if deidentify:
         mu += "__DE-IDENTIFIED__: the names and national identifiers of real (Non-Machine) end users have been scrubbed from this report. VistA-specific IENs are left to identify such users for those with access to the system.\n\n"
             
-    tbl = MarkdownTable([":Type", "Users", "SignOns"], includeNo=False)
+    tbl = MarkdownTable(["Type", "Users", "SignOns"], includeNo=False)
     tbl.addRow([
         "Total Users (200 entries)", 
         len(userInfoByUserRef),
@@ -147,6 +158,16 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
             type3_081["_total"]
         )
     ])
+    tbl.addRow([
+        "Active __Uncategorized__", 
+        reportAbsAndPercent(len(classification["activeNotCategorizedUserRefs"]), len(classification["activeUserRefs"])),
+        reportAbsAndPercent(
+            sum(st3_081ByUserRef[userRef]["_total"] for userRef
+ in classification["activeNotCategorizedUserRefs"]),
+            type3_081["_total"]
+        )
+    ])
+    
     mu += "User type signon summary ...\n\n"
     mu += tbl.md() + "\n\n"
     
@@ -177,8 +198,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         classification["warningsByUserRef"],
         userInfoByUserRef,
         st3_081ByUserRef,
-        type3_081["_total"],
-        keyColsOnly = deidentify
+        type3_081["_total"]
     )
 
     mu += webReportNonProxyMachineUsers(
@@ -186,8 +206,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         classification["warningsByUserRef"],
         userInfoByUserRef,
         type3_081["_total"],
-        st3_081ByUserRef,
-        keyColsOnly = deidentify
+        st3_081ByUserRef
     )
 
     mu += webReportRemoteUsers(
@@ -198,8 +217,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         type3_081["_total"], 
         st3_081ByUserRef,
         classification["remoteExcludeReasonCount"], 
-        deidentify,
-        keyColsOnly = deidentify
+        deidentify
     )
         
     # NOTE: far too crude now - need breaks
@@ -210,12 +228,21 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         userInfoByUserRef, 
         type3_081["_total"], 
         st3_081ByUserRef,
-        deidentify,
-        keyColsOnly = deidentify
+        deidentify
     )
+    
+    mu += webReportUnclassifiedUsers(
+        classification["activeNotCategorizedUserRefs"], 
+        classification["warningsByUserRef"],
+        len(classification["activeUserRefs"]), 
+        userInfoByUserRef, 
+        type3_081["_total"], 
+        st3_081ByUserRef,
+        deidentify
+    )    
 
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
-    reportFile = userSiteDir + ("user.md" if not deidentify else "userDeId.md")
+    reportFile = userSiteDir + ("user2.md" if not deidentify else "user2DeId.md")
     print("Writing report {}".format(reportFile))
     open(reportFile, "w").write(mu)
 
@@ -243,22 +270,21 @@ https://github.com/vistadataproject/RPCDefinitionToolkit/issues/44
 Note: tied up with vistalink and two step of connector (with station number
 lookup) and then switch to local user <------------ see if two step means two sign ons?
 """
-def webReportProxyUsers(activeProxyUserRefs, warningsByUserRef, userInfoByUserRef, st3_081ByUserRef, totalSignOns, keyColsOnly=False):
+def webReportProxyUsers(activeProxyUserRefs, warningsByUserRef, userInfoByUserRef, st3_081ByUserRef, totalSignOns):
 
     totalProxySignOnCount = sum(st3_081ByUserRef[userRef]["_total"] for userRef
  in activeProxyUserRefs)
     mu = """## Proxy Users
     
 There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (user_class is \"CONNECTOR PROXY\") with <span class='yellowIt'>{}</span> signons. All user records have _access_, _verify_ and lack a social while their signons have _LOA_ 2 and don't have \"remote_...\" properties (ie/ CPRS-like combo). These signons probably happen over __VistALink__ and not the plain RPC Broker ...\n\n""".format(len(activeProxyUserRefs), reportAbsAndPercent(totalProxySignOnCount, totalSignOns))
-    cols = [":Name [IEN]", "Entered", "SignOns", "Period", "\# IPs"] if keyColsOnly else [":Name [IEN]", "Entered", ":PMO", ":SMOs", ":Keys", "SignOns", "Period", "\# IPs", "Duration", ":Unexpected"] 
-    tbl = MarkdownTable(cols)
+    tbl = MarkdownTable(["Name [IEN]", "Entered", "PMO", "SMOs", "Keys", "SOs", "Period", "\# IPs", "Duration", "Unexpected"])
     allSTs = []
     for userRef in sorted(activeProxyUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef] 
         st = st3_081ByUserRef[userRef]
         allSTs.append(st)
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
-        if "duration" in st:
+        if "duration" in st and "byValueCount" in st["duration"]: # TODO: remove and FORCE all to have duration once redo E
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -269,28 +295,18 @@ There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (use
         else:
             durMU = ""
         unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-        if keyColsOnly:
-            row = [
-                muUserRef(userRef),
-                userInfo["date_entered"] if "date_entered" in userInfo else "", 
-                reportAbsAndPercent(st["_total"], totalProxySignOnCount),
-                muSignOnPeriod(st),
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5) 
-            ]
-        else:           
-            row = [
-                muUserRef(userRef),
-                userInfo["date_entered"] if "date_entered" in userInfo else "", 
-                pmoMU,
-                smosMU,
-                keysMU,
-                reportAbsAndPercent(st["_total"], totalProxySignOnCount),
-                muSignOnPeriod(st),
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
-                durMU,
-                unexpectedMU
-            ]
-        tbl.addRow(row)
+        tbl.addRow([
+            muUserRef(userRef),
+            userInfo["date_entered"] if "date_entered" in userInfo else "", 
+            pmoMU,
+            smosMU,
+            keysMU,
+            reportAbsAndPercent(st["_total"], totalProxySignOnCount),
+            muSignOnPeriod(st),
+            muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
+            durMU,
+            unexpectedMU
+        ])
     mu += tbl.md() + "\n\n"
             
     return mu
@@ -312,7 +328,7 @@ CVIX_USER_SSNIEN = "200:217122" # expect 2001 too; 2006_95 << sign ons
 ...
 and fixed IP and LOA 1 usually
 """
-def webReportNonProxyMachineUsers(activeNonProxyMachineUserRefs, warningsByUserRef, userInfoByUserRef, totalSignOns, st3_081ByUserRef, keyColsOnly=False):
+def webReportNonProxyMachineUsers(activeNonProxyMachineUserRefs, warningsByUserRef, userInfoByUserRef, totalSignOns, st3_081ByUserRef):
 
     totalNonProxyMachineSignOnCount = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNonProxyMachineUserRefs)
     mu = """## (Non Proxy) Machine Users
@@ -323,21 +339,20 @@ Besides the _DoD User_, there are <span class='yellowIt'>{:,}</span> active __No
     )
 
     # To add: workstation_name - take first part? ipv4_address    
-    cols = [":Name [IEN]", "Entered", "SSN", "SignOns", "Period", "Remote Station Id(s)", "Remote IEN(s)", ":IPs"] if keyColsOnly else [":Name [IEN]", "Entered", "SSN", ":SMOs", "SignOns", "Period", "Remote Station Id(s)", "Remote IEN(s)", ":IPs", "Duration", ":Unexpected"]
-    tbl = MarkdownTable(cols)
+    tbl = MarkdownTable(["Name [IEN]", "Entered", "SSN", "SMOs", "SOs", "Period", "Remote Station Id(s)", "Remote IEN(s)", "IPs", "Duration", "Unexpected"])
     for userRef in sorted(activeNonProxyMachineUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef]
         st = st3_081ByUserRef[userRef]
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
         unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
         if "remote_user_ien" in st:
-            if len(st["remote_user_ien"]["byValueCount"].keys()) > 5:
-                remoteIENsMU = "_#{:,}_".format(len(st["remote_user_ien"]["byValueCount"].keys()))
+            if len(st["remote_user_ien"]["byValueCount"]) > 5:
+                remoteIENsMU = "_#{:,}_".format(len(st["remote_user_ien"]["byValueCount"]))
             else:
-                remoteIENsMU = "/".join(st["remote_user_ien"]["byValueCount"].keys())
+                remoteIENsMU = "/".join(st["remote_user_ien"]["byValueCount"])
         else:
             remoteIENsMU = ""
-        if "duration" in st:
+        if "duration" in st and "byValueCount" in st["duration"]: # TODO: remove
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -347,34 +362,20 @@ Besides the _DoD User_, there are <span class='yellowIt'>{:,}</span> active __No
                 durMU = muSeconds(singleValue(st, "duration"))
         else:
             durMU = ""
-        if keyColsOnly:
-            row = [
-                muUserRef(userRef),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
-                reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
-                muSignOnPeriod(st),
-                # NO remote app?
-                "/".join(st["remote_station_id"]["byValueCount"].keys()) if "remote_station_id" in st else "",
-                remoteIENsMU,
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5)            
-            ]
-        else:
-            row = [
-                muUserRef(userRef),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
-                smosMU,
-                reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
-                muSignOnPeriod(st),
-                # NO remote app?
-                "/".join(st["remote_station_id"]["byValueCount"].keys()) if "remote_station_id" in st else "",
-                remoteIENsMU,
-                muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
-                durMU,
-                unexpectedMU      
-            ]
-        tbl.addRow(row)
+        tbl.addRow([
+            muUserRef(userRef),
+            userInfo["date_entered"] if "date_entered" in userInfo else "",
+            "__{}__".format(userInfo["ssn"]) if "ssn" in userInfo else "",
+            smosMU,
+            reportAbsAndPercent(st["_total"], totalNonProxyMachineSignOnCount),
+            muSignOnPeriod(st),
+            # NO remote app?
+            "/".join(st["remote_station_id"]["byValueCount"]) if "remote_station_id" in st else "",
+            remoteIENsMU,
+            muBVC(st["ipv4_address"]["byValueCount"], countOnlyIfOver=5),
+            durMU,
+            unexpectedMU      
+        ])
     mu += tbl.md() + "\n\n"
         
     return mu
@@ -398,7 +399,7 @@ def webReportDoDUser(userRef, userInfo, totalSignons, st, warnings, deidentify):
     
 One special non proxy, machine user, the __\"DoD User\"__ is used for JLV DoD access and for access by a number of other applications ...\n\n"""
     
-    tbl = MarkdownTable([":Property", ":Value"], includeNo=False)
+    tbl = MarkdownTable(["Property", "Value"])
     
     tbl.addRow(["Name \[IEN\]", muUserRef(userRef)])
     tbl.addRow(["Date Entered", userInfo["date_entered"] if "date_entered" in userInfo else ""])
@@ -439,7 +440,7 @@ Every VistA has __Postmaster__, (one of) the first user in the system ...
 
 """
     
-    tbl = MarkdownTable([":Property", ":Value"], includeNo=False)
+    tbl = MarkdownTable(["Property", "Value"])
     
     tbl.addRow(["Name \[IEN\]", muUserRef(userRef)])
     tbl.addRow(["Date Entered", userInfo["date_entered"] if "date_entered" in userInfo else ""])
@@ -485,7 +486,7 @@ TODO: (review the issues in TK too)
 - manilla called out; 200 too?
 fmrepo/user/catag (move over)
 """
-def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, remoteExcludeReasonCount, deidentify, keyColsOnly=False):
+def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, remoteExcludeReasonCount, deidentify):
 
     remoteSignOnCountsByUserRef = dict((userRef, st3_081ByUserRef[userRef]["_total"]) for userRef in activeRemoteUserRefs)
     totalRemoteSignOns = sum(remoteSignOnCountsByUserRef[userRef] for userRef in remoteSignOnCountsByUserRef)
@@ -493,7 +494,7 @@ def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUse
 
     print("About to combine lot's of remote sts (takes time) ...")
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeRemoteUserRefs], forceCountProps=["remote_station_id", "remote_app", "division"])[0]
-    print ("... end combo'ing")
+    print("... end combo'ing")
     
     mu = """## Remote Users
     
@@ -510,7 +511,7 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
         len(remoteStationBVC),
         len(set(stId[0:3] for stId in remoteStationBVC))
     )    
-    tbl = MarkdownTable([":Remote Station Id", "Count of Remote SignOns"])
+    tbl = MarkdownTable(["Remote Station Id", "Count of Remote SignOns"])
     for i, stationId in enumerate(sorted(remoteStationBVC, key=lambda x: remoteStationBVC[x], reverse=True), 1):
         if i > 10:
             break
@@ -538,18 +539,22 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
             return sid 
         if len(stationIdVC) == 1:
             return "{}:{}".format(
-                muTripler(stationIdVC.keys()[0]), 
-                ienVC.keys()[0]
+                muTripler(list(stationIdVC)[0]), 
+                list(ienVC)[0]
             )
         if len(ienVC) == 1:
             return "{}:{}".format(
-                "/".join([muTripler(sid) for sid in stationIdVC.keys()]), 
-                ienVC.keys()[0]
+                "/".join([muTripler(sid) for sid in stationIdVC]), 
+                list(ienVC)[0]
             )
         # TODO: match counts in both to assemble id
         return ""
     mu += "The following shows the top 50 Remote Users ...\n\n"
-    cols = [":Name [IEN]", "Entry", "Remote Id(s)", "SignOns", "Period", "Days", ":Remote Apps"] if keyColsOnly else [":Name [IEN]", "Entry", "Remote Id(s)", "SignOns", "Period", "Days", ":Remote Apps", ":Options", "Duration", ":Unexpected"]   
+    cols = ["Name [IEN]", "Entry", "Remote Id(s)", "SOs", "Period", "Days", "Remote Apps", "Options", "Duration"]
+    useUnexpected = False
+    if sum(1 for userRef in activeRemoteUserRefs if userRef in warningsByUserRef):
+        useUnexpected = True
+        cols.append("Unexpected") 
     tbl = MarkdownTable(cols)
     for i, userRef in enumerate(sorted(activeRemoteUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True), 1):
         if i > 50:
@@ -564,7 +569,7 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
             remote_app_count["UNIDENTIFIED"] = no_remote_app_count
         remoteAppMU = ", ".join(["{} ({})".format(remote_app.split(" [")[0], remote_app_count[remote_app]) for remote_app in sorted(remote_app_count, key=lambda x: remote_app_count[x], reverse=True)])
         wdCntr = expandByWeekDay(st)
-        if "duration" in st:
+        if "duration" in st and "byValueCount" in st["duration"]:
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -574,41 +579,41 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
                 durMU = muSeconds(singleValue(st, "duration"))
         else:
             durMU = ""
-        if keyColsOnly:
-            row = [ 
-                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                muRemoteIds(st),
-                reportAbsAndPercent(st["_total"], totalRemoteSignOns),
-                muSignOnPeriod(st),
+        row = [
+            muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+            userInfo["date_entered"] if "date_entered" in userInfo else "",
+            muRemoteIds(st),
+            reportAbsAndPercent(st["_total"], totalRemoteSignOns),
+            muSignOnPeriod(st),
             ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
-                remoteAppMU            
-            ]
-        else:
-            row = [
-                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                muRemoteIds(st),
-                reportAbsAndPercent(st["_total"], totalRemoteSignOns),
-                muSignOnPeriod(st),
-            ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
-                remoteAppMU,
-                smosMU,
-                durMU,
-                "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-            ]
+            remoteAppMU,
+            smosMU,
+            durMU
+        ]
+        if useUnexpected:
+            unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
+            row.append(unexpectedMU)
         tbl.addRow(row)
     mu += tbl.md() + "\n\n"
         
     return mu
     
+def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
+
+    totalUnclassfiedSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNotCategorizedUserRefs)
+    mu = """## Unclassified Users
+    
+There are <span class='yellowIt'>{}</span> active unclassified Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeNotCategorizedUserRefs), totalActiveUsers), reportAbsAndPercent(totalUnclassfiedSignOns, totalSignOns))
+
+    return mu
+    
 """
 TODO: 
 - broad: FAR TOO catch all -- why does PUG have so many more in Pug than ALake? 
   - and device: ssh vs other
 - exclude from remote on MAND_PROPS and ALLOWED_PROPS as opposed to 0's ie/ 0's in here PLUS station_no in locals
 """
-def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify, keyColsOnly=False):
+def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
 
     totalLocalSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeLocalUserRefs)
     mu = """## Local Users
@@ -616,8 +621,8 @@ def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers
 There are <span class='yellowIt'>{}</span> active Local Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeLocalUserRefs), totalActiveUsers), reportAbsAndPercent(totalLocalSignOns, totalSignOns))
 
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeLocalUserRefs], forceCountProps=["division"])[0]
-    mu += "Local users select <span class='yellowIt'>{:,}</span> divisions ...\n\n".format(len(comboST["division"]["byValueCount"]))
-    tbl = MarkdownTable([":Division", "Count %"])
+    mu += "Local users select <span class='yellowIt'>{:,}</span> divisions (SHOULD RESTRICT THIS SET) ...\n\n".format(len(comboST["division"]["byValueCount"]))
+    tbl = MarkdownTable(["Division", "Count %"])
     for divisionRef in sorted(comboST["division"]["byValueCount"], key=lambda x: comboST["division"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(re.sub(r'4\-', '', divisionRef)), 
@@ -626,7 +631,7 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     mu += tbl.md() + "\n\n"
     
     mu += "And don't just sign on through device 0 ...\n\n"
-    tbl = MarkdownTable([":Device", "Count %"])
+    tbl = MarkdownTable(["Device", "Count %"])
     for device in sorted(comboST["device"]["byValueCount"], key=lambda x: comboST["device"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(device), 
@@ -636,50 +641,55 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     
     # TODO: will break on these 
     mu += "And have multiple levels of assurance ...\n\n"
-    tbl = MarkdownTable([":LOA", "Count %"])
+    tbl = MarkdownTable(["LOA", "Count %"])
     for loa in sorted(comboST["level_of_assurance"]["byValueCount"], key=lambda x: comboST["level_of_assurance"]["byValueCount"][x], reverse=True):
         tbl.addRow([
-            "__{}__".format(loa), 
-            reportAbsAndPercent(comboST["level_of_assurance"]["byValueCount"][loa], comboST["_total"])
+            "__{}__".format(device), 
+            reportAbsAndPercent(comboST["level_of_assurance"]["byValueCount"][device], comboST["_total"])
         ])
     mu += tbl.md() + "\n\n"
     
     SUPERUSER_KEYS = ["XUMGR"] # removing XUPROG
     superUserRefs = set(userRef for userRef in activeLocalUserRefs if "keys" in userInfoByUserRef[userRef] and len(set(SUPERUSER_KEYS).intersection(set(userInfoByUserRef[userRef]["keys"]))))
     mu += "<span class='yellowIt'>{:,}</span> Local Users are __Superusers__ (those with key {}) ...\n\n".format(len(superUserRefs), "|".join(SUPERUSER_KEYS))
-    cols = [":Name [IEN]", "Entry", ":Title", "SignOns", "Period", "Days"] if keyColsOnly else [":Name [IEN]", "Entry", ":Title", "SignOns", "Period", "Days", ":PMO", ":SMOs", ":Keys", ":Unexpected"]
+    cols = ["Name [IEN]", "Entry", "Title", "SOs", "Period", "Days", "PMO", "SMOs", "Keys"]
+    useUnexpected = False
+    if sum(1 for userRef in superUserRefs if userRef in warningsByUserRef):
+        useUnexpected = True
+        cols.append("Unexpected") 
     tbl = MarkdownTable(cols)
     for userRef in sorted(superUserRefs, key=lambda x: st3_081ByUserRef[x]["_total"], reverse=True):
         userInfo = userInfoByUserRef[userRef]
         st = st3_081ByUserRef[userRef]
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
         wdCntr = expandByWeekDay(st)
-        if keyColsOnly:
-            row = [
-                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                userInfo["title"] if "title" in userInfo else "",
-                reportAbsAndPercent(st["_total"], totalLocalSignOns),
-                muSignOnPeriod(st),
-                ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)])            
-            ]
-        else:
-            row = [
-                muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
-                userInfo["date_entered"] if "date_entered" in userInfo else "",
-                userInfo["title"] if "title" in userInfo else "",
-                reportAbsAndPercent(st["_total"], totalLocalSignOns),
-                muSignOnPeriod(st),
-                ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
-                pmoMU,
-                smosMU,
-                keysMU,
-                "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
-            ]
+        row = [
+            muUserRef(userRef, ssn="NO SSN" if "ssn" not in userInfo else userInfo["ssn"], deidentify=deidentify),
+            userInfo["date_entered"] if "date_entered" in userInfo else "",
+            userInfo["title"] if "title" in userInfo else "",
+            reportAbsAndPercent(st["_total"], totalLocalSignOns),
+            muSignOnPeriod(st),
+            ", ".join(["{} [{}]".format("__{}__".format(day) if i < 5 else day, wdCntr[day]) for i, day in enumerate(wdCntr)]),
+            pmoMU,
+            smosMU,
+            keysMU
+        ]
+        if useUnexpected:
+            unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
+            row.append(unexpectedMU)
         tbl.addRow(row)
     mu += tbl.md() + "\n\n"
         
     return mu
+    
+def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify):
+
+    totalUnclassfiedSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNotCategorizedUserRefs)
+    mu = """## Unclassified Users
+    
+There are <span class='yellowIt'>{}</span> active unclassified Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeNotCategorizedUserRefs), totalActiveUsers), reportAbsAndPercent(totalUnclassfiedSignOns, totalSignOns))
+
+    return mu
        
 # TODO: change to calc length formally in case of gaps 
 def muSignOnPeriod(st):
@@ -687,7 +697,7 @@ def muSignOnPeriod(st):
         soPeriodMU = "EVERY MONTH"
     elif st["date_time"]["firstCreateDate"].split("T")[0] == st["date_time"]["lastCreateDate"].split("T")[0]:
         soPeriodMU = st["date_time"]["lastCreateDate"].split("T")[0]
-    elif re.search(r'\-', st["date_time"]["byValueCount"].keys()[0]): # months
+    elif re.search(r'\-', list(st["date_time"]["byValueCount"])[0]): # months
         soPeriodMU = "{} - {} ({})".format(st["date_time"]["firstCreateDate"].split("T")[0], st["date_time"]["lastCreateDate"].split("T")[0], len(st["date_time"]["byValueCount"]))
     else:      
         soPeriodMU = "{} - {}".format(st["date_time"]["firstCreateDate"].split("T")[0], st["date_time"]["lastCreateDate"].split("T")[0])
@@ -732,7 +742,7 @@ def muUserRef(userRef, ssn=None, deidentify=False):
     
 def tblByWeekDay(st):
     ocntr = expandByWeekDay(st, fullDays=True)
-    tbl = MarkdownTable([":Day", "Total", "Average"], includeNo=False)
+    tbl = MarkdownTable(["Day", "Total", "Average"], includeNo=False)
     for i, day in enumerate(ocntr):
         avg = ocntr[day].split("/")[1]
         avgMU = "" if avg == "0" else avg
@@ -770,6 +780,7 @@ def expandByWeekDay(st, fullDays=False):
     return cntr
     
 def muSeconds(seconds):
+    seconds = int(seconds)
     (days, remainder) = divmod(seconds, 86400)
     (hours, remainder) = divmod(remainder, 3600)
     (minutes, seconds) = divmod(remainder, 60)
@@ -797,17 +808,16 @@ def muSeconds(seconds):
                
 def main():
 
-    assert(sys.version_info >= (2,7))
-    
-    if len(sys.argv) < 2:
-        print "need to specify station # ex/ 442 - exiting"
-        return
-        
-    stationNo = sys.argv[1]
-    
+    assert sys.version_info >= (3, 4)
+
+    try:
+        stationNo = sys.argv[1]
+    except IndexError:
+        raise SystemExit("Usage _EXE_ STATIONNO")
+
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     if not os.path.isdir(userSiteDir):
-        raise Exception("Expect User Site to already exist with its basic contents")
+        raise SystemExit("Expect User Site to already exist with its basic contents")
     
     if len(sys.argv) == 3:
         deidentify = sys.argv[2]
