@@ -248,7 +248,7 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         
         userId = "{} [{}]".format(resource["label"], resource["_id"])
                 
-        info = {"hasProps": sorted([key for key in resource.keys() if key not in set(["_id", "label", "type"])])}
+        info = {"hasProps": sorted([key for key in resource if key not in set(["_id", "label", "type"])])}
         
         if "creator" in resource:
             if resource["creator"]["id"] == "200-0":
@@ -467,7 +467,7 @@ def reduce44(stationNo, useRF=True): # do from RF
             red[mprop].add(pref)
         red[mprop] = list(red[mprop])
         return red
-    institutionsByHPrefix = defaultdict(set) # for mapping
+    institutionsByHPrefix = defaultdict(lambda: Counter()) # for mapping
     cnt = 0
     for resource in filteredResultIterator(stationNo, "44", useRF=useRF):
         ien = resource["_id"].split("-")[1]
@@ -476,6 +476,7 @@ def reduce44(stationNo, useRF=True): # do from RF
         cnt += 1
         if (cnt % 200) == 0:
             print("processed another 200 to {:,}".format(cnt))
+            sys.stdout.flush()
         for litProp in ["abbreviation", "patient_friendly_name", "physical_location", "length_of_appointment", "default_appointment_type", "noncount_clinic_y_or_n", "clinic_meets_at_this_facility"]: # includes enum
             if litProp not in resource:
                 continue
@@ -491,8 +492,9 @@ def reduce44(stationNo, useRF=True): # do from RF
             ref = "{} [{}]".format(refv["label"], refv["id"].split("-")[1])
             red[refProp] = ref
             if refProp == "institution":
-                hprefix = re.split(r'[ \-\/]', resource["name"])[0]
-                institutionsByHPrefix[hprefix].add(ref)
+                hprefix = re.sub(r'^ZZ', '', re.split(r'[ \-\/]', resource["name"])[0])
+                if len(hprefix):
+                    institutionsByHPrefix[hprefix][ref] += 1
         # Mult Availability - note many exs where no appts
         if "availability" in resource: # want to see if always there if appointments too
             red["has_availability"] = True
@@ -502,8 +504,23 @@ def reduce44(stationNo, useRF=True): # do from RF
             countWProvider += 1
         copyMProp(resource, red, "privileged_user") # the clerks etc
         # Note: appointment FLIPPED/REDUCED so won't know that?
-        
-    # Fill in missing instit links based on prefix mappings from those with an instit
+    
+    """    
+    Fill in missing instit links based on prefix mappings from those with an instit
+    
+    Note that based on error'ed assignment of prefixes, > 1 institution my be assigned
+    a prefix. We will take majority rule so it there is > one then 
+    """
+    ninstitutionsByHPrefix = {}
+    for hprefix in institutionsByHPrefix:
+        if len(list(institutionsByHPrefix)) == 1:
+            ninstitutionsByHPrefix[hprefix] = list(institutionsByHPrefix)[0]
+            continue
+        useTotal = sum(institutionsByHPrefix[hprefix][instit] for instit in institutionsByHPrefix[hprefix])
+        for instit in institutionsByHPrefix[hprefix]:
+            if float(institutionsByHPrefix[hprefix][instit])/useTotal >= 0.75:
+                ninstitutionsByHPrefix[hprefix] = instit
+                break
     noMissing = 0
     noFilled = 0
     for ien in infosByIEN:
@@ -512,8 +529,8 @@ def reduce44(stationNo, useRF=True): # do from RF
             continue
         noMissing += 1
         hprefix = re.split(r'[ \-\/]', info["label"])[0]
-        if hprefix in institutionsByHPrefix and len(institutionsByHPrefix[hprefix]) == 1:
-            info["institution"] = list(institutionsByHPrefix[hprefix])[0]
+        if hprefix in ninstitutionsByHPrefix:
+            info["institution"] = ninstitutionsByHPrefix[hprefix]
             noFilled += 1
     print("Of {:,} locations missing an institution link, {:,} were filled by heuristic".format(noMissing, noFilled))
         
